@@ -1,4 +1,4 @@
-# test_client.py -- Compatibilty tests for git client.
+# test_client.py -- Compatibility tests for git client.
 # Copyright (C) 2010 Google, Inc.
 #
 # Dulwich is dual-licensed under the Apache License, Version 2.0 and the GNU
@@ -18,10 +18,10 @@
 # License, Version 2.0.
 #
 
-"""Compatibilty tests between the Dulwich client and the cgit server."""
+"""Compatibility tests between the Dulwich client and the cgit server."""
 
 import copy
-from io import BytesIO
+import http.server
 import os
 import select
 import signal
@@ -31,38 +31,27 @@ import sys
 import tarfile
 import tempfile
 import threading
-
+from contextlib import suppress
+from io import BytesIO
 from urllib.parse import unquote
 
-import http.server
+from dulwich import client, file, index, objects, protocol, repo
+from dulwich.tests import SkipTest, expectedFailure
 
-from dulwich import (
-    client,
-    file,
-    index,
-    protocol,
-    objects,
-    repo,
-)
-from dulwich.tests import (
-    SkipTest,
-    expectedFailure,
-)
-from dulwich.tests.compat.utils import (
+from .utils import (
+    _DEFAULT_GIT,
     CompatTestCase,
     check_for_daemon,
     import_repo_to_dir,
     rmtree_ro,
     run_git_or_fail,
-    _DEFAULT_GIT,
 )
 
-
 if sys.platform == "win32":
     import ctypes
 
 
-class DulwichClientTestBase(object):
+class DulwichClientTestBase:
     """Tests for client/server compatibility."""
 
     def setUp(self):
@@ -84,10 +73,10 @@ class DulwichClientTestBase(object):
                 self.assertReposEqual(src, dest)
 
     def _client(self):
-        raise NotImplementedError()
+        raise NotImplementedError
 
     def _build_path(self):
-        raise NotImplementedError()
+        raise NotImplementedError
 
     def _do_send_pack(self):
         c = self._client()
@@ -248,12 +237,10 @@ class DulwichClientTestBase(object):
                 dest.refs.set_if_equals(r[0], None, r[1])
             self.assertEqual(
                 dest.get_shallow(),
-                set(
-                    [
-                        b"35e0b59e187dd72a0af294aedffc213eaa4d03ff",
-                        b"514dc6d3fbfe77361bcaef320c4d21b72bc10be9",
-                    ]
-                ),
+                {
+                    b"35e0b59e187dd72a0af294aedffc213eaa4d03ff",
+                    b"514dc6d3fbfe77361bcaef320c4d21b72bc10be9",
+                },
             )
 
     def test_repeat(self):
@@ -331,13 +318,13 @@ class DulwichClientTestBase(object):
             sendrefs[b"refs/heads/abranch"] = b"00" * 20
             del sendrefs[b"HEAD"]
 
-            def gen_pack(have, want, ofs_delta=False):
+            def gen_pack(have, want, ofs_delta=False, progress=None):
                 return 0, []
 
             c = self._client()
             self.assertEqual(dest.refs[b"refs/heads/abranch"], dummy_commit)
             c.send_pack(self._build_path("/dest"), lambda _: sendrefs, gen_pack)
-            self.assertFalse(b"refs/heads/abranch" in dest.refs)
+            self.assertNotIn(b"refs/heads/abranch", dest.refs)
 
     def test_send_new_branch_empty_pack(self):
         with repo.Repo(os.path.join(self.gitroot, "dest")) as dest:
@@ -346,7 +333,7 @@ class DulwichClientTestBase(object):
             dest.refs[b"refs/heads/abranch"] = dummy_commit
             sendrefs = {b"refs/heads/bbranch": dummy_commit}
 
-            def gen_pack(have, want, ofs_delta=False):
+            def gen_pack(have, want, ofs_delta=False, progress=None):
                 return 0, []
 
             c = self._client()
@@ -406,11 +393,9 @@ class DulwichTCPClientTest(CompatTestCase, DulwichClientTestBase):
             ctypes.windll.kernel32.TerminateProcess(handle, -1)
             ctypes.windll.kernel32.CloseHandle(handle)
         else:
-            try:
+            with suppress(OSError):
                 os.kill(pid, signal.SIGKILL)
                 os.unlink(self.pidfile)
-            except (OSError, IOError):
-                pass
         self.process.wait()
         self.process.stdout.close()
         self.process.stderr.close()
@@ -429,8 +414,13 @@ class DulwichTCPClientTest(CompatTestCase, DulwichClientTestBase):
         def test_fetch_pack_no_side_band_64k(self):
             DulwichClientTestBase.test_fetch_pack_no_side_band_64k(self)
 
+    def test_send_remove_branch(self):
+        # This test fails intermittently on my machine, probably due to some sort
+        # of race condition. Probably also related to #1015
+        self.skipTest("skip flaky test; see #1015")
 
-class TestSSHVendor(object):
+
+class TestSSHVendor:
     @staticmethod
     def run_command(
         host,
@@ -444,7 +434,7 @@ class TestSSHVendor(object):
         cmd = cmd.split("-", 1)
         path = path.replace("'", "")
         p = subprocess.Popen(
-            cmd + [path],
+            [*cmd, path],
             bufsize=0,
             stdin=subprocess.PIPE,
             stdout=subprocess.PIPE,
@@ -508,7 +498,7 @@ class GitHTTPRequestHandler(http.server.SimpleHTTPRequestHandler):
         # Let's be quiet, the test suite is noisy enough already
         pass
 
-    def run_backend(self):  # noqa: C901
+    def run_backend(self):
         """Call out to git http-backend."""
         # Based on CGIHTTPServer.CGIHTTPRequestHandler.run_cgi:
         # Copyright (c) 2001-2010 Python Software Foundation;
@@ -602,9 +592,23 @@ class GitHTTPRequestHandler(http.server.SimpleHTTPRequestHandler):
         try:
             nbytes = int(length)
         except (TypeError, ValueError):
-            nbytes = 0
-        if self.command.lower() == "post" and nbytes > 0:
-            data = self.rfile.read(nbytes)
+            nbytes = -1
+        if self.command.lower() == "post":
+            if nbytes > 0:
+                data = self.rfile.read(nbytes)
+            elif self.headers.get("transfer-encoding") == "chunked":
+                chunks = []
+                while True:
+                    line = self.rfile.readline()
+                    length = int(line.rstrip(), 16)
+                    chunk = self.rfile.read(length + 2)
+                    chunks.append(chunk[:-2])
+                    if length == 0:
+                        break
+                data = b"".join(chunks)
+                env["CONTENT_LENGTH"] = str(len(data))
+            else:
+                raise AssertionError
         else:
             data = None
             env["CONTENT_LENGTH"] = "0"
@@ -620,20 +624,18 @@ class GitHTTPRequestHandler(http.server.SimpleHTTPRequestHandler):
 
 
 class HTTPGitServer(http.server.HTTPServer):
-
     allow_reuse_address = True
 
-    def __init__(self, server_address, root_path):
+    def __init__(self, server_address, root_path) -> None:
         http.server.HTTPServer.__init__(self, server_address, GitHTTPRequestHandler)
         self.root_path = root_path
         self.server_name = "localhost"
 
     def get_url(self):
-        return "http://%s:%s/" % (self.server_name, self.server_port)
+        return f"http://{self.server_name}:{self.server_port}/"
 
 
 class DulwichHttpClientTest(CompatTestCase, DulwichClientTestBase):
-
     min_git_version = (1, 7, 0, 2)
 
     def setUp(self):
