@@ -34,12 +34,26 @@ from dulwich.tests import TestCase
 
 from ..errors import NotTreeError
 from ..index import commit_tree
-from ..object_store import (DiskObjectStore, MemoryObjectStore,
-                            ObjectStoreGraphWalker, OverlayObjectStore,
-                            commit_tree_changes, iter_tree_contents, peel_sha,
-                            read_packs_file, tree_lookup_path)
-from ..objects import (S_IFGITLINK, Blob, EmptyFileException,
-                       SubmoduleEncountered, Tree, TreeEntry, sha_to_hex)
+from ..object_store import (
+    DiskObjectStore,
+    MemoryObjectStore,
+    ObjectStoreGraphWalker,
+    OverlayObjectStore,
+    commit_tree_changes,
+    iter_tree_contents,
+    peel_sha,
+    read_packs_file,
+    tree_lookup_path,
+)
+from ..objects import (
+    S_IFGITLINK,
+    Blob,
+    EmptyFileException,
+    SubmoduleEncountered,
+    Tree,
+    TreeEntry,
+    sha_to_hex,
+)
 from ..pack import REF_DELTA, write_pack_objects
 from ..protocol import DEPTH_INFINITE
 from .utils import build_pack, make_object, make_tag
@@ -70,27 +84,21 @@ class ObjectStoreTests:
         self.store.add_object(testobject)
         refs = {b"refs/heads/foo": testobject.id}
         with patch.object(self.store, "_get_depth", return_value=1) as m:
-            self.assertEqual(
-                [], self.store.determine_wants_all(refs, depth=0)
-            )
+            self.assertEqual([], self.store.determine_wants_all(refs, depth=0))
             self.assertEqual(
                 [testobject.id],
                 self.store.determine_wants_all(refs, depth=DEPTH_INFINITE),
             )
             m.assert_not_called()
 
-            self.assertEqual(
-                [], self.store.determine_wants_all(refs, depth=1)
-            )
+            self.assertEqual([], self.store.determine_wants_all(refs, depth=1))
             m.assert_called_with(testobject.id)
             self.assertEqual(
                 [testobject.id], self.store.determine_wants_all(refs, depth=2)
             )
 
     def test_get_depth(self):
-        self.assertEqual(
-            0, self.store._get_depth(testobject.id)
-        )
+        self.assertEqual(0, self.store._get_depth(testobject.id))
 
         self.store.add_object(testobject)
         self.assertEqual(
@@ -427,7 +435,7 @@ class DiskObjectStoreTests(PackBasedObjectStoreTests, TestCase):
         self.assertEqual(oct(mode), packmode)
 
     def test_corrupted_object_raise_exception(self):
-        """Corrupted sha1 disk file should raise specific exception"""
+        """Corrupted sha1 disk file should raise specific exception."""
         self.store.add_object(testobject)
         self.assertEqual(
             (Blob.type_num, b"yummy data"), self.store.get_raw(testobject.id)
@@ -584,8 +592,12 @@ class TreeLookupPathTests(TestCase):
     def test_lookup_submodule(self):
         tree_lookup_path(self.get_object, self.tree_id, b"d")[1]
         self.assertRaises(
-            SubmoduleEncountered, tree_lookup_path, self.get_object,
-            self.tree_id, b"d/a")
+            SubmoduleEncountered,
+            tree_lookup_path,
+            self.get_object,
+            self.tree_id,
+            b"d/a",
+        )
 
     def test_lookup_nonexistent(self):
         self.assertRaises(
